#include "tt_squirrel_exception.h"

#include "tt_squirrel_native_api.h"

#ifndef REAL_FUNC_NAME_OF
#define REAL_FUNC_NAME_OF( f ) (#f)
#endif

#ifndef FUNC_NAME_OF
#ifdef DEBUG
#  define FUNC_NAME_OF( f ) ((void)(f), (*(f)) == ((f)), #f)
#else
#  define FUNC_NAME_OF( f ) (#f)
#endif
#endif

using namespace TtSquirrel;


// -- NativeAPI ----------------------------------------------------------
NativeAPI::NativeAPI( HSQUIRRELVM& vm ) :
vm_( vm )
{
}


HSQUIRRELVM
NativeAPI::Open( int stack_size )
{
  return sq_open( stack_size );
}

void
NativeAPI::Close( void )
{
  sq_close( vm_ );
}

void
NativeAPI::PushRootTable( void )
{
  sq_pushroottable( vm_ );
}

void
NativeAPI::PushInteger( int n )
{
  sq_pushinteger( vm_, n );
}

void
NativeAPI::PushFloat( float f )
{
  sq_pushfloat( vm_, f );
}

void
NativeAPI::PushString( const std::string& str )
{
  sq_pushstring( vm_, str.c_str(), str.size() );
}

void
NativeAPI::PushBoolean( bool flag )
{
  sq_pushbool( vm_, flag ? SQTrue : SQFalse );
}

void
NativeAPI::PushUserPointer( SQUserPointer pointer )
{
  sq_pushuserpointer( vm_, pointer );
}

void
NativeAPI::PushNull( void )
{
  sq_pushnull( vm_ );
}


ObjectType
NativeAPI::GetType( int index )
{
  return sq_gettype( vm_, index );
}


int
NativeAPI::GetAsInteger( int index )
{
  int tmp;
  this->ResultTest( sq_getinteger( vm_, index, &tmp ), FUNC_NAME_OF( sq_getinteger ) );
  return tmp;
}

float
NativeAPI::GetAsFloat( int index )
{
  float tmp;
  this->ResultTest( sq_getfloat( vm_, index, &tmp ), FUNC_NAME_OF( sq_getfloat ) );
  return tmp;

}

std::string
NativeAPI::GetAsString( int index )
{
  const char* tmp;
  this->ResultTest( sq_getstring( vm_, index, &tmp ), FUNC_NAME_OF( sq_getstring ) );
  return tmp;
}

bool
NativeAPI::GetAsBoolean( int index )
{
  SQBool tmp;
  this->ResultTest( sq_getbool( vm_, index, &tmp ), FUNC_NAME_OF( sq_getbool ) );
  return tmp == SQTrue;
}

SQUserPointer
NativeAPI::GetAsUserPointer( int index )
{
  SQUserPointer tmp;
  this->ResultTest( sq_getuserpointer( vm_, index, &tmp ), FUNC_NAME_OF( sq_getuserpointer ) );
  return tmp;
}

HSQUIRRELVM
NativeAPI::GetAsThread( int index )
{
  HSQUIRRELVM tmp;
  this->ResultTest( sq_getthread( vm_, index, &tmp ), FUNC_NAME_OF( sq_getthread ) );
  return tmp;
}

std::pair<SQUserPointer, SQUserPointer>
NativeAPI::GetAsUserData( int index )
{
  SQUserPointer pointer;
  SQUserPointer type_tag;
  this->ResultTest( sq_getuserdata( vm_, index, &pointer, &type_tag ), FUNC_NAME_OF( sq_getuserdata ) );
  return std::pair<SQUserPointer, SQUserPointer>( pointer, type_tag );
}


void
NativeAPI::PopTop( void )
{
  sq_poptop( vm_ );
}


void
NativeAPI::Pop( int count )
{
  sq_pop( vm_, count );
}


void
NativeAPI::Push( int index )
{
  sq_push( vm_, index );
}

void
NativeAPI::SetTop( int top_index )
{
  sq_settop( vm_, top_index );
}

void
NativeAPI::ResizeStackSize( int top_index )
{
  sq_settop( vm_, top_index );
}

void
NativeAPI::Remove( int index )
{
  sq_remove( vm_, index );
}


int
NativeAPI::GetTop( void )
{
  return sq_gettop( vm_ );
}

int
NativeAPI::GetTopIndex( void )
{
  return this->GetTop();
}

void
NativeAPI::Call( int param_count, bool need_return_value , bool raise_exception_in_error )
{
  this->ResultTest( sq_call( vm_, param_count,
                       need_return_value ? SQTrue : SQFalse,
                       raise_exception_in_error ? SQTrue : SQFalse ),
              FUNC_NAME_OF( sq_call ) );
}



// Table
void
NativeAPI::NewTable( void )
{
  sq_newtable( vm_ );
}

void
NativeAPI::CreateSlot( int index )
{
  this->ResultTest( sq_createslot( vm_, index ), REAL_FUNC_NAME_OF( sq_createslot ) );
}

void
NativeAPI::SetDelegate( int index )
{
  this->ResultTest( sq_setdelegate( vm_, index ), FUNC_NAME_OF( sq_setdelegate ) );
}

void
NativeAPI::GetDelegate( int index )
{
  this->ResultTest( sq_getdelegate( vm_, index ), FUNC_NAME_OF( sq_getdelegate ) );
}

// Array
void
NativeAPI::NewArray( int size )
{
  sq_newarray( vm_, size );
}

void
NativeAPI::ArrayAppend( int index )
{
  this->ResultTest( sq_arrayappend( vm_, index ), FUNC_NAME_OF( sq_arrayappend ) );
}

void
NativeAPI::ArrayPop( int index, bool pushval )
{
  this->ResultTest( sq_arraypop( vm_, index, pushval ), FUNC_NAME_OF( sq_arraypop ) );
}

void
NativeAPI::ArrayResize( int index, int new_size )
{
  this->ResultTest( sq_arrayresize( vm_, index, new_size ), FUNC_NAME_OF( sq_arrayresize ) );
}

void
NativeAPI::ArrayReverse( int index )
{
  this->ResultTest( sq_arrayreverse( vm_, index ), FUNC_NAME_OF( sq_arrayreverse ) );
}

void
NativeAPI::ArrayRemove( int index, int item_index )
{
  this->ResultTest( sq_arrayremove( vm_, index, item_index ), FUNC_NAME_OF( sq_arrayremove ) );
}

void
NativeAPI::ArrayInsert( int index, int destination_position )
{
  this->ResultTest( sq_arrayinsert( vm_, index, destination_position ), FUNC_NAME_OF( sq_arrayinsert ) );
}


// Table or Array
int
NativeAPI::GetSize( int index )
{
  return sq_getsize( vm_, index );
}

void
NativeAPI::Get( int index )
{
  this->ResultTest( sq_get( vm_, index ), FUNC_NAME_OF( sq_get ) );
}

void
NativeAPI::Set( int index )
{
  this->ResultTest( sq_set( vm_, index ), FUNC_NAME_OF( sq_set ) );
}

void
NativeAPI::RawGet( int index )
{
  this->ResultTest( sq_rawget( vm_, index ), FUNC_NAME_OF( sq_rawget ) );
}

void
NativeAPI::RagSet( int index )
{
  this->ResultTest( sq_rawset( vm_, index ), FUNC_NAME_OF( sq_rawset ) );
}

void
NativeAPI::Next( int index )
{
  this->ResultTest( sq_next( vm_, index ), FUNC_NAME_OF( sq_next ) );
}


HSQUIRRELVM
NativeAPI::NewThread( int initial_stack_size )
{
  return sq_newthread( vm_, initial_stack_size );
}


void
NativeAPI::SetErrorHandler( void )
{
  sq_seterrorhandler( vm_ );
}

void
NativeAPI::SetForeignPointer( SQUserPointer pointer )
{
  sq_setforeignptr( vm_, pointer );
}

SQUserPointer
NativeAPI::GetForeignPointer( void )
{
  return sq_getforeignptr( vm_ );
}

void
NativeAPI::SetPrintFunction( SQPRINTFUNCTION print_function )
{
  sq_setprintfunc( vm_, print_function );
}

SQPRINTFUNCTION
NativeAPI::GetPrintFunction( void )
{
  return sq_getprintfunc( vm_ );
}

void
NativeAPI::SuspendVM( void )
{
  this->ResultTest( sq_suspendvm( vm_ ), FUNC_NAME_OF( sq_suspendvm ) );
}

void
NativeAPI::SuspendVirtualMachine( void )
{
  this->SuspendVM();
}


void
NativeAPI::WakeupVM( bool resumedret, bool retval, bool raiseerror, bool throwerror )
{
  this->ResultTest( sq_wakeupvm( vm_, resumedret, retval, raiseerror, throwerror ), FUNC_NAME_OF( sq_wakeupvm ) );
}

void
NativeAPI::WakeupVirtualMachine( bool resumedret, bool retval, bool raiseerror, bool throwerror )
{
  this->WakeupVM( resumedret, retval, raiseerror, throwerror );
}

int
NativeAPI::GetVMState( void )
{
  return sq_getvmstate( vm_ );
}

int
NativeAPI::GetVirtualMachineState( void )
{
  return this->GetVMState();
}

void
NativeAPI::Compile( SQLEXREADFUNC read, SQUserPointer pointer, const std::string& source_name, bool raise_error )
{
  this->ResultTest( sq_compile( vm_, read, pointer, source_name.c_str(), raise_error ), FUNC_NAME_OF( sq_compile ) );
}

void
NativeAPI::CompileBuffer( const std::string& s, int size, const std::string& source_name, bool raise_error )
{
  this->ResultTest( sq_compilebuffer( vm_, s.c_str(), size, source_name.c_str(), raise_error ), FUNC_NAME_OF( sq_compilebuffer ) );
}

void
NativeAPI::EnableDebugInfo( bool enable )
{
  sq_enabledebuginfo( vm_, enable );
}

void
NativeAPI::NotifyAllExceptions( bool enable )
{
  sq_notifyallexceptions( vm_, enable );
}

void
NativeAPI::SetCompilerErrorHandler( SQCOMPILERERROR f )
{
  sq_setcompilererrorhandler( vm_, f );
}


void
NativeAPI::ReserveStack( int size )
{
  sq_reservestack( vm_, size );
}

int
NativeAPI::Cmp( void )
{
  return sq_cmp( vm_ );
}

int
NativeAPI::Compare( void )
{
  return this->Cmp();
}

void
NativeAPI::Move( HSQUIRRELVM destination, int index )
{
  sq_move( destination, vm_, index );
}

void
NativeAPI::MoveObjectBetweenVirtualMachine( HSQUIRRELVM destination, int index )
{
  this->Move( destination, index );
}

SQUserPointer
NativeAPI::NewUserData( unsigned int size )
{
  return sq_newuserdata( vm_, size );
}


void
NativeAPI::NewClosure( SQFUNCTION func, unsigned int nfreevars )
{
  sq_newclosure( vm_, func, nfreevars );
}

void
NativeAPI::SetParamsCheck( int nparamscheck, const std::string& typemask )
{
  this->ResultTest( sq_setparamscheck( vm_, nparamscheck, typemask.c_str() ), FUNC_NAME_OF( sq_setparamscheck ) );
}

void
NativeAPI::BindEnv( int index )
{
  this->ResultTest( sq_bindenv( vm_, index ), FUNC_NAME_OF( sq_bindenv ) );
}

void
NativeAPI::BindEnvironment( int index )
{
  this->BindEnv( index );
}

void
NativeAPI::GetBase( int index )
{
  this->ResultTest( sq_getbase( vm_, index ), FUNC_NAME_OF( sq_getbase ) );
}

bool
NativeAPI::InstanceOf( void )
{
  return sq_instanceof( vm_ );
}

void
NativeAPI::ToString( int index )
{
  sq_tostring( vm_, index );
}

bool
NativeAPI::ToBoolean( int index )
{
  unsigned int tmp;
  sq_tobool( vm_, index, &tmp );
  return tmp;
}


void
NativeAPI::SetTypeTag( int index, SQUserPointer type_tag )
{
  this->ResultTest( sq_settypetag( vm_, index, type_tag ), FUNC_NAME_OF( sq_settypetag ) );
}

SQUserPointer
NativeAPI::GetTypeTag( int index )
{
  SQUserPointer type_tag;
  this->ResultTest( sq_gettypetag( vm_, index, &type_tag ), FUNC_NAME_OF( sq_gettypetag ) );
  return type_tag;
}

void
NativeAPI::SetReleaseHook( int index, SQRELEASEHOOK hook )
{
  sq_setreleasehook( vm_, index, hook );
}

char*
NativeAPI::GetScratchPad( int min_size )
{
  return sq_getscratchpad( vm_, min_size );
}

void
NativeAPI::GetFunctionInfo( int index, SQFunctionInfo* fi )
{
  this->ResultTest( sq_getfunctioninfo( vm_, index, fi ), FUNC_NAME_OF( sq_getfunctioninfo ) );
}

void
NativeAPI::GetClosureInfo( int index, unsigned int* nparams, unsigned int* nfreevars )
{
  this->ResultTest( sq_getclosureinfo( vm_, index, nparams, nfreevars ), FUNC_NAME_OF( sq_getclosureinfo ) );
}

void
NativeAPI::SetNativeClosureName( int index, const std::string& name )
{
  this->ResultTest( sq_setnativeclosurename( vm_, index, name.c_str() ), FUNC_NAME_OF( sq_setnativeclosurename ) );
}

void
NativeAPI::SetInstanceUp( int index, SQUserPointer pointer )
{
  this->ResultTest( sq_setinstanceup( vm_, index, pointer ), FUNC_NAME_OF( sq_setinstanceup ) );
}

SQUserPointer
NativeAPI::GetInstanceUp( int index, SQUserPointer type_tag )
{
  SQUserPointer tmp;
  this->ResultTest( sq_getinstanceup( vm_, index, &tmp, type_tag ), FUNC_NAME_OF( sq_getinstanceup ) );
  return tmp;
}

void
NativeAPI::SetClassUserDataSize( int index, int user_data_size )
{
  this->ResultTest( sq_setclassudsize( vm_, index, user_data_size ), FUNC_NAME_OF( sq_setclassudsize ) );
}

void
NativeAPI::NewClass( bool hasbase )
{
  this->ResultTest( sq_newclass( vm_, hasbase ), FUNC_NAME_OF( sq_newclass ) );
}

void
NativeAPI::CreateInstance( int index )
{
  this->ResultTest( sq_createinstance( vm_, index ), FUNC_NAME_OF( sq_createinstance ) );
}

void
NativeAPI::SetAttributes( int index )
{
  this->ResultTest( sq_setattributes( vm_, index ), FUNC_NAME_OF( sq_setattributes ) );
}

void
NativeAPI::GetAttributes( int index )
{
  this->ResultTest( sq_getattributes( vm_, index ), FUNC_NAME_OF( sq_getattributes ) );
}

void
NativeAPI::GetClass( int index )
{
  this->ResultTest( sq_getclass( vm_, index ), FUNC_NAME_OF( sq_getclass ) );
}

void
NativeAPI::WeakReference( int index )
{
  sq_weakref( vm_, index );
}



void
NativeAPI::GetDefaultDelegate( ObjectType type )
{
  this->ResultTest( sq_getdefaultdelegate( vm_, type.ToValue() ), FUNC_NAME_OF( sq_getdefaultdelegate ) );
}

void
NativeAPI::PushRegistryTable( void )
{
  sq_pushregistrytable( vm_ );
}

void
NativeAPI::PushConstTable( void )
{
  sq_pushconsttable( vm_ );
}

void
NativeAPI::SetRootTable( void )
{
  this->ResultTest( sq_setroottable( vm_ ), FUNC_NAME_OF( sq_setroottable ) );
}

void
NativeAPI::SetConstTable( void )
{
  this->ResultTest( sq_setconsttable( vm_ ), FUNC_NAME_OF( sq_setconsttable ) );
}

void
NativeAPI::NewSlot( int index, bool is_static )
{
  this->ResultTest( sq_newslot( vm_, index, is_static ), FUNC_NAME_OF( sq_newslot ) );
}

void
NativeAPI::DeleteSlot( int index, bool push_value )
{
  this->ResultTest( sq_deleteslot( vm_, index, push_value ), FUNC_NAME_OF( sq_deleteslot ) );
}

void
NativeAPI::RawDeleteSlot( int index, bool pushval )
{
  this->ResultTest( sq_rawdeleteslot( vm_, index, pushval ), FUNC_NAME_OF( sq_rawdeleteslot ) );
}



void
NativeAPI::Clone( int index )
{
  this->ResultTest( sq_clone( vm_, index ), FUNC_NAME_OF( sq_clone ) );
}

void
NativeAPI::SetFreeVariable( int index, unsigned int nval )
{
  this->ResultTest( sq_setfreevariable( vm_, index, nval ), FUNC_NAME_OF( sq_setfreevariable ) );
}


void
NativeAPI::GetWeakReferenceValue( int index )
{
  this->ResultTest( sq_getweakrefval( vm_, index ), FUNC_NAME_OF( sq_getweakrefval ) );
}


void
NativeAPI::Clear( int index )
{
  this->ResultTest( sq_clear( vm_, index ), FUNC_NAME_OF( sq_clear ) );
}

void
NativeAPI::Resume( bool retval, bool raiseerror )
{
  this->ResultTest( sq_resume( vm_, retval, raiseerror ), FUNC_NAME_OF( sq_resume ) );
}

const std::string
NativeAPI::GetLocal( unsigned int level, unsigned int index )
{
  return sq_getlocal( vm_, level, index );
}

const std::string
NativeAPI::GetFreeVariable( unsigned int index, unsigned int nval )
{
  return sq_getfreevariable( vm_, index, nval );
}

SQRESULT
NativeAPI::ThrowError( const std::string& err )
{
  return sq_throwerror( vm_, err.c_str() );
}

void
NativeAPI::ResetError( void )
{
  sq_reseterror( vm_ );
}

void
NativeAPI::GetLastError( void )
{
  sq_getlasterror( vm_ );
}

void
NativeAPI::GetStackObject( int index, HSQOBJECT* object )
{
  this->ResultTest( sq_getstackobj( vm_, index, object ), FUNC_NAME_OF( sq_getstackobj ) );
}

void
NativeAPI::PushObject( HSQOBJECT object )
{
  sq_pushobject( vm_, object );
}

void
NativeAPI::AddReference( HSQOBJECT* object )
{
  sq_addref( vm_, object );
}

bool
NativeAPI::Release( HSQOBJECT* object )
{
  return sq_release( vm_, object );
}

void
NativeAPI::ResetObject( HSQOBJECT* object )
{
  sq_resetobject( object );
}


std::string
NativeAPI::ObjectToString( HSQOBJECT* object )
{
  return sq_objtostring( object );
}


bool
NativeAPI::ObjectToBoolean( HSQOBJECT* object )
{
  return sq_objtobool( object );
}

int
NativeAPI::ObjectToInteger( HSQOBJECT* object )
{
  return sq_objtointeger( object );
}

float
NativeAPI::ObjectToFloat( HSQOBJECT* object )
{
  return sq_objtofloat( object );
}


SQUserPointer
NativeAPI::GetObjectTypeTag( HSQOBJECT* object )
{
  SQUserPointer type_tag;
  if ( SQ_FAILED( sq_getobjtypetag( object, &type_tag ) ) ) {
    throw NativeAPIException(
      FUNC_NAME_OF( sq_getobjtypetag ), "object is not instance or user data or class" );
  }
  return type_tag;
}

int
NativeAPI::CollectGarbage( void )
{
  return sq_collectgarbage( vm_ );
}

void
NativeAPI::WriteClosure( SQWRITEFUNC write_function, SQUserPointer user_pointer )
{
  this->ResultTest( sq_writeclosure( vm_, write_function, user_pointer ), FUNC_NAME_OF( sq_writeclosure ) );
}

void
NativeAPI::ReadClosure( SQREADFUNC read_function, SQUserPointer user_pointer )
{
  this->ResultTest( sq_readclosure( vm_, read_function, user_pointer ), FUNC_NAME_OF( sq_readclosure ) );
}

void*
NativeAPI::Malloc( unsigned int size )
{
  return sq_malloc( size );
}

void*
NativeAPI::Realloc( void* pointer, unsigned int old_size, unsigned int new_size )
{
  return sq_realloc( pointer, old_size, new_size );
}

void
NativeAPI::Free( void* pointer, unsigned int size )
{
  sq_free( pointer, size );
}



void
NativeAPI::StackInfomations( int level, SQStackInfos* stack_info )
{
  this->ResultTest( sq_stackinfos( vm_, level, stack_info ), FUNC_NAME_OF( sq_stackinfos ) );
}

void
NativeAPI::SetDebugHook( void )
{
  sq_setdebughook( vm_ );
}



// -- Standard -----------------------------------------------------------

NativeAPI::Standard::Standard( HSQUIRRELVM& vm ) :
vm_( vm )
{
}


#include "sqstdaux.h"

void
NativeAPI::Standard::SetErrorHandlers( void )
{
  sqstd_seterrorhandlers( vm_ );
}

void
NativeAPI::Standard::PrintCallStack( void )
{
  sqstd_printcallstack( vm_ );
}


#include "sqstdblob.h"

SQUserPointer
NativeAPI::Standard::CreateBlob( int size )
{
  return sqstd_createblob( vm_, size );
}

SQUserPointer
NativeAPI::Standard::GetBlob( int index )
{
  SQUserPointer tmp;
  NativeAPI( vm_ ).ResultTest( sqstd_getblob( vm_, index, &tmp ), FUNC_NAME_OF( sqstd_getblob ) );
  return tmp;
}

int
NativeAPI::Standard::GetBlobSize( int index )
{
  return sqstd_getblobsize( vm_, index );
}

void
NativeAPI::Standard::RegisterBlobLibrary( void )
{
  NativeAPI( vm_ ).ResultTest( sqstd_register_bloblib( vm_ ), FUNC_NAME_OF( sqstd_register_bloblib ) );
}


#include "sqstdio.h"

SQFILE
NativeAPI::Standard::FOpen( const std::string& filename, const std::string& mode )
{
  return sqstd_fopen( filename.c_str(), mode.c_str() );
}

int
NativeAPI::Standard::FRead( SQUserPointer buffer, int size, int count, SQFILE file )
{
  return sqstd_fread( buffer, size, count, file );
}

int
NativeAPI::Standard::FWrite( SQUserPointer buffer, int size, int count, SQFILE file )
{
  return sqstd_fwrite( buffer, size, count, file );
}

int
NativeAPI::Standard::FSeek( SQFILE file, int offset, int origin )
{
  return sqstd_fseek( file, offset, origin );
}

int
NativeAPI::Standard::FTell( SQFILE file )
{
  return sqstd_ftell( file );
}

int
NativeAPI::Standard::FFlush( SQFILE file )
{
  return sqstd_fflush( file );
}

int
NativeAPI::Standard::Fclose( SQFILE file )
{
  return sqstd_fclose( file );
}

int
NativeAPI::Standard::FEof( SQFILE file )
{
  return sqstd_feof( file );
}


void
NativeAPI::Standard::CreateFile( SQFILE file, bool own )
{
  NativeAPI( vm_ ).ResultTest( sqstd_createfile( vm_, file, own ), FUNC_NAME_OF( sqstd_createfile ) );
}

SQFILE
NativeAPI::Standard::GetFile( int index )
{
  SQFILE tmp;
  NativeAPI( vm_ ).ResultTest( sqstd_getfile( vm_, index, &tmp ), FUNC_NAME_OF( sqstd_getfile ) );
  return tmp;
}

void
NativeAPI::Standard::LoadFile( const std::string& filename, bool printerror )
{
  NativeAPI( vm_ ).ResultTest( sqstd_loadfile( vm_, filename.c_str(), printerror ), FUNC_NAME_OF( sqstd_loadfile ) );
}

void
NativeAPI::Standard::DoFile( const std::string& filename, bool retval, bool printerror )
{
  NativeAPI( vm_ ).ResultTest( sqstd_dofile( vm_, filename.c_str(), retval, printerror ), FUNC_NAME_OF( sqstd_dofile ) );
}

void
NativeAPI::Standard::WriteClosureToFile( const std::string& filename )
{
  NativeAPI( vm_ ).ResultTest( sqstd_writeclosuretofile( vm_, filename.c_str() ), FUNC_NAME_OF( sqstd_writeclosuretofile ) );
}

void
NativeAPI::Standard::RegisterIOLibrary( void )
{
  NativeAPI( vm_ ).ResultTest( sqstd_register_iolib( vm_ ), FUNC_NAME_OF( sqstd_register_iolib ) );
}


#include "sqstdmath.h"

void
NativeAPI::Standard::RegisterMathLibrary( void )
{
  NativeAPI( vm_ ).ResultTest( sqstd_register_mathlib( vm_ ), FUNC_NAME_OF( sqstd_register_mathlib ) );
}


#include "sqstdstring.h"

SQRex*
NativeAPI::Standard::RegexCompile( const std::string& pattern, std::string error )
{
  const char* tmp;
  SQRex* rex;
  rex = sqstd_rex_compile( pattern.c_str(), &tmp );
  error = tmp;
  return rex;
}

void
NativeAPI::Standard::RegexFree( SQRex* exp )
{
  sqstd_rex_free( exp );
}

bool
NativeAPI::Standard::RegexMatch( SQRex* exp, const std::string& text )
{
  return sqstd_rex_match( exp, text.c_str() );
}

bool
NativeAPI::Standard::RegexSearch( SQRex* exp, const char* text, const char** out_begin, const char** out_end )
{
  return sqstd_rex_search( exp, text, out_begin, out_end );
}

bool
NativeAPI::Standard::RexSearchRange( SQRex* exp, const char* text_begin, const char* text_end, const char** out_begin, const char** out_end )
{
  return sqstd_rex_searchrange( exp, text_begin, text_end, out_begin, out_end );
}

int
NativeAPI::Standard::RexGetSubExpressionCount( SQRex* exp )
{
  return sqstd_rex_getsubexpcount( exp );
}

bool
NativeAPI::Standard::RexGetSubExpression( SQRex* exp, int n, SQRexMatch* subexp )
{
  return sqstd_rex_getsubexp( exp, n, subexp );
}


void
NativeAPI::Standard::Format( int format_string_index, int* out_string_length, char** output )
{
  NativeAPI( vm_ ).ResultTest( sqstd_format( vm_, format_string_index, out_string_length, output ), FUNC_NAME_OF( sqstd_format ) );
}

void
NativeAPI::Standard::RegisterStringLibrary( void )
{
  NativeAPI( vm_ ).ResultTest( sqstd_register_stringlib( vm_ ), FUNC_NAME_OF( sqstd_register_stringlib ) );
}


#include "sqstdsystem.h"

int
NativeAPI::Standard::RegisterSystemLibrary( void )
{
  return sqstd_register_systemlib( vm_ );
}



NativeAPI::Standard
NativeAPI::Std( void )
{
  return Standard( vm_ );
}



void
NativeAPI::ResultTest( SQRESULT result, const char* function_name )
{
  if ( SQ_FAILED( result ) ) {
    this->GetLastError();
    std::string reason = this->GetAsString( -1 );
    this->PopTop();
    throw NativeAPIException( function_name, reason );
  }
}


HSQUIRRELVM
NativeAPI::GetHandle( void )
{
  return vm_;
}
