#ifndef TT_SQUIRREL_NATIVE_API_H_
#define TT_SQUIRREL_NATIVE_API_H_

#include <string>

#include "squirrel.h"
#include "sqstdio.h"
#include "sqstdstring.h"

#include "tt_squirrel_object_type.h"


namespace TtSquirrel {
  // -- NativeAPI --------------------------------------------------------
  class NativeAPI {
  public:
    explicit NativeAPI( HSQUIRRELVM& vm );

  public:
    // Virtual Machine
    static HSQUIRRELVM Open( int stack_size );
    void               Close( void );

    SQUserPointer   GetForeignPointer( void );
    SQPRINTFUNCTION GetPrintFunction( void );
    int             GetVMState( void );
    int             GetVirtualMachineState( void ); // alias GetVMState
    void            Move( HSQUIRRELVM destination, int index );
    void            MoveObjectBetweenVirtualMachine( HSQUIRRELVM destination, int index ); // alias Move
    HSQUIRRELVM     NewThread( int initial_stack_size );
    void            PushRootTable( void );
    void            PushRegistryTable( void );
    void            SetErrorHandler( void );
    void            SetForeignPointer( SQUserPointer pointer );
    void            SetPrintFunction( SQPRINTFUNCTION print_function );
    void            SetRootTable( void );
    void            SuspendVM( void );
    void            SuspendVirtualMachine( void ); // alias SuspendVM
    void            WakeupVM( bool resumedret, bool retval, bool raiseerror, bool throwerror );
    void            WakeupVirtualMachine( bool resumedret, bool retval, bool raiseerror, bool throwerror ); // alias WakeupVM
    int             CollectGarbage( void );

    // Compiler
    void Compile( SQLEXREADFUNC read, SQUserPointer pointer, const std::string& source_name, bool raise_error );
    void CompileBuffer( const std::string& s, int size, const std::string& source_name, bool raise_error );
    void EnableDebugInfo( bool enable );
    void NotifyAllExceptions( bool enable );
    void SetCompilerErrorHandler( SQCOMPILERERROR f );

    // Stack Operations;
    int  Cmp( void );
    int  Compare( void ); // alias Cmp
    int  GetTop( void );
    int  GetTopIndex( void ); // alias GetTop
    void Pop( int count );
    void PopTop( void );
    void Push( int index );
    void Remove( int index );
    void ReserveStack( int size );
    void SetTop( int top_index );
    void ResizeStackSize( int top_index ); // alias SetTop

    // Calls;
    void              Call( int param_count, bool need_return_value = true, bool raise_exception_in_error = true );
    void              GetLastError( void );
    const std::string GetLocal( unsigned int level, unsigned int index );
    void              ResetError( void );
    void              Resume( bool retval, bool raiseerror );
    SQRESULT          ThrowError( const std::string& err );

    // Object creation and handling;
    void           BindEnv( int index );
    void           BindEnvironment( int index ); // alias BindEnv
    void           CreateInstance( int index );
    void           GetClosureInfo( int index, unsigned int* nparams, unsigned int* nfreevars );
    SQUserPointer  GetInstanceUp( int index, SQUserPointer type_tag );

    int            GetAsInteger( int index );
    float          GetAsFloat( int index );
    std::string    GetAsString( int index );
    bool           GetAsBoolean( int index );
    SQUserPointer  GetAsUserPointer( int index );
    HSQUIRRELVM    GetAsThread( int index );
    std::pair<SQUserPointer, SQUserPointer> GetAsUserData( int index );

    char*         GetScratchPad( int min_size );
    ObjectType    GetType( int index );
    SQUserPointer GetTypeTag( int index );
    void          NewClass( bool hasbase );
    void          NewClosure( SQFUNCTION func, unsigned int nfreevars );
    SQUserPointer NewUserData( unsigned int size );
    void          PushInteger( int n );
    void          PushFloat( float f );
    void          PushString( const std::string& str );
    void          PushBoolean( bool flag );
    void          PushUserPointer( SQUserPointer pointer );
    void          PushNull( void );
    void          SetInstanceUp( int index, SQUserPointer pointer );
    void          SetNativeClosureName( int index, const std::string& name );
    void          SetParamsCheck( int nparamscheck, const std::string& typemask );
    void          SetReleaseHook( int index, SQRELEASEHOOK hook );
    void          SetTypeTag( int index, SQUserPointer type_tag );
    bool          ToBoolean( int index );
    void          ToString( int index );

    // Objects manipulation;
    void              Clone( int index );
    void              GetAttributes( int index );
    void              GetBase( int index );
    void              GetClass( int index );
    void              GetDelegate( int index );
    void              GetDefaultDelegate( ObjectType type );
    const std::string GetFreeVariable( unsigned int index, unsigned int nval );
    void              GetWeakReferenceValue( int index );
    bool              InstanceOf( void );
    void              SetAttributes( int index );
    void              SetDelegate( int index );
    void              SetFreeVariable( int index, unsigned int nval );
    void              WeakReference( int index );

    // Table
    void NewTable( void );
    void CreateSlot( int index );
    void DeleteSlot( int index, bool push_value );
    void RawDeleteSlot( int index, bool pushval );
    void NewSlot( int index, bool is_static );
    int  GetSize( int index );
    void Get( int index );
    void Set( int index );
    void RawGet( int index );
    void RagSet( int index );
    void Next( int index );

    // Array
    void NewArray( int size );
    void ArrayAppend( int index );
    void ArrayPop( int index, bool push_value );
    void ArrayResize( int index, int new_size );
    void ArrayReverse( int index );
    void ArrayRemove( int index, int item_index );
    void ArrayInsert( int index, int destination_position );

    // Bytecode serialization ;
    void WriteClosure( SQWRITEFUNC write_function, SQUserPointer user_pointer );
    void ReadClosure( SQREADFUNC read_function, SQUserPointer user_pointer );

    // Raw object handling;
    void          AddReference( HSQOBJECT* object );
    void          GetStackObject( int index, HSQOBJECT* object );
    void          PushObject( HSQOBJECT object );
    bool          Release( HSQOBJECT* object );
    static void          ResetObject( HSQOBJECT* object );
    static std::string   ObjectToString( HSQOBJECT* object );
    static bool          ObjectToBoolean( HSQOBJECT* object );
    static int           ObjectToInteger( HSQOBJECT* object );
    static float         ObjectToFloat( HSQOBJECT* object );
    static SQUserPointer GetObjectTypeTag( HSQOBJECT* object );

    // Debug interface
    void GetFunctionInfo( int index, SQFunctionInfo* fi );
    void SetDebugHook( void );
    void StackInfomations( int level, SQStackInfos* stack_info );

    // unknown
    void SetClassUserDataSize( int index, int user_data_size );
    void PushConstTable( void );
    void SetConstTable( void );
    void Clear( int index );

    // memory allocation
    static void* Malloc( unsigned int size );
    static void* Realloc( void* pointer, unsigned int old_size, unsigned int new_size );
    static void  Free( void* pointer, unsigned int size );

    // -- Standard -------------------------------------------------------
    class Standard {
      friend class NativeAPI;

    private:
      explicit Standard( HSQUIRRELVM& vm );

    public:
      // aux
      void SetErrorHandlers( void );
      void PrintCallStack( void );

      // blob
      SQUserPointer CreateBlob( int size );
      SQUserPointer GetBlob( int index );
      int           GetBlobSize( int index );
      void          RegisterBlobLibrary( void );

      // IO
      SQFILE FOpen( const std::string& filename, const std::string& mode );
      int    FRead( SQUserPointer buffer, int size, int count, SQFILE file );
      int    FWrite( SQUserPointer buffer, int size, int count, SQFILE file );
      int    FSeek( SQFILE file, int offset, int origin );
      int    FTell( SQFILE file );
      int    FFlush( SQFILE file );
      int    Fclose( SQFILE file );
      int    FEof( SQFILE file );
      void   CreateFile( SQFILE file, bool own );
      SQFILE GetFile( int index );
      void   LoadFile( const std::string& filename, bool printerror );
      void   DoFile( const std::string& filename, bool retval, bool printerror );
      void   WriteClosureToFile( const std::string& filename );
      void   RegisterIOLibrary( void );

      // math
      void RegisterMathLibrary( void );

      // regular expression
      SQRex* RegexCompile( const std::string& pattern, std::string error );
      void   RegexFree( SQRex* exp );
      bool   RegexMatch( SQRex* exp, const std::string& text );
      bool   RegexSearch( SQRex* exp, const char* text, const char** out_begin, const char** out_end );
      bool   RexSearchRange( SQRex* exp, const char* text_begin, const char* text_end, const char** out_begin, const char** out_end );
      int    RexGetSubExpressionCount( SQRex* exp );
      bool   RexGetSubExpression( SQRex* exp, int n, SQRexMatch* subexp );

      // stringvoid
      void Format( int format_string_index, int* out_string_length, char** output );
      void RegisterStringLibrary( void );

      // system
      int RegisterSystemLibrary( void );

    private:
      HSQUIRRELVM& vm_;
    };

    Standard Std( void );

    void ResultTest( SQRESULT result, const char* function_name );

    HSQUIRRELVM GetHandle( void );

  private:
    HSQUIRRELVM& vm_;
  };
}

#endif // TT_SQUIRREL_NATIVE_API_H_
