#ifndef TT_SQUIRREL_OPERATION_H_
#define TT_SQUIRREL_OPERATION_H_

#include <string>

namespace TtSquirrel {
  class Operation {
  public:
    virtual void operator ()( void ) const = 0;

    // -- Value ----------------------------------------------------------
    template <class TYPE>
    struct Value {
      Value( TYPE value ) : value_( value ) {}
      TYPE value_;
    };

    template <class TYPE>
    struct Value<TYPE&> {
      Value( TYPE& value ) : value_( value ) {}
      TYPE& value_;
    };
  };

  template <class OBJECT, class FUNCTION>
  class OperationArg0 : public Operation {
  public:
    explicit OperationArg0( const OBJECT& object, FUNCTION function ) :
    object_( object ), function_( function ) {}
    void operator ()( void ) const {
      (const_cast<OBJECT&>( object_ ).*function_)();
    }
  private:
    const OBJECT& object_;
    FUNCTION      function_;
  };

  template <class OBJECT, class FUNCTION, class ARG1>
  class OperationArg1 : public Operation {
  public:
    explicit OperationArg1( const OBJECT& object, FUNCTION function,
                            Operation::Value<ARG1> arg1 ) :
    object_( object ), function_( function ),
    arg1_( arg1 ) {}
    void operator ()( void ) const {
      (const_cast<OBJECT&>( object_ ).*function_)( arg1_.value_ );
    }
  private:
    const OBJECT&          object_;
    FUNCTION               function_;
    Operation::Value<ARG1> arg1_;
  };

  template <class OBJECT, class FUNCTION, class ARG1, class ARG2>
  class OperationArg2 : public Operation {
  public:
    explicit OperationArg2( const OBJECT& object, FUNCTION function,
                            Operation::Value<ARG1> arg1,
                            Operation::Value<ARG2> arg2 ) :
    object_( object ), function_( function ),
    arg1_( arg1 ), arg2_( arg2 ) {}
    void operator ()( void ) const {
      (const_cast<OBJECT&>( object_ ).*function_)( arg1_.value_, arg2_.value_ );
    }
  private:
    const OBJECT&          object_;
    FUNCTION               function_;
    Operation::Value<ARG1> arg1_;
    Operation::Value<ARG2> arg2_;
  };

  template <class OBJECT, class FUNCTION, class ARG1, class ARG2, class ARG3>
  class OperationArg3 : public Operation {
  public:
    explicit OperationArg3( const OBJECT& object, FUNCTION function,
                            Operation::Value<ARG1> arg1,
                            Operation::Value<ARG2> arg2,
                            Operation::Value<ARG3> arg3 ) :
    object_( object ), function_( function ),
    arg1_( arg1 ), arg2_( arg2 ), arg3_( arg3 ) {}
    void operator ()( void ) const {
      (const_cast<OBJECT&>( object_ ).*function_)( arg1_.value_, arg2_.value_, arg3_.value_ );
    }
  private:
    const OBJECT&          object_;
    FUNCTION               function_;
    Operation::Value<ARG1> arg1_;
    Operation::Value<ARG2> arg2_;
    Operation::Value<ARG3> arg3_;
  };

  template <class OBJECT, class FUNCTION, class ARG1, class ARG2, class ARG3, class ARG4>
  class OperationArg4 : public Operation {
  public:
    explicit OperationArg4( const OBJECT& object, FUNCTION function,
                            Operation::Value<ARG1> arg1,
                            Operation::Value<ARG2> arg2,
                            Operation::Value<ARG3> arg3,
                            Operation::Value<ARG4> arg4 ) :
    object_( object ), function_( function ),
    arg1_( arg1 ), arg2_( arg2 ), arg3_( arg3 ), arg4_( arg4 ) {}
    void operator ()( void ) const {
      (const_cast<OBJECT&>( object_ ).*function_)( arg1_.value_, arg2_.value_, arg3_.value_, arg4_.value_ );
    }
  private:
    const OBJECT&          object_;
    FUNCTION               function_;
    Operation::Value<ARG1> arg1_;
    Operation::Value<ARG2> arg2_;
    Operation::Value<ARG3> arg3_;
    Operation::Value<ARG4> arg4_;
  };

  // -- MakeOperation ------------------------------------------------------
  template <class TYPE>
  Operation::Value<TYPE> Arg( TYPE value ) {
    return Operation::Value<TYPE>( value );
  }

  template <class TYPE>
  Operation::Value<TYPE&> ArgRef( TYPE& value ) {
    return Operation::Value<TYPE&>( value );
  }

  template <class OBJECT, class FUNCTION>
  OperationArg0<OBJECT, FUNCTION> MakeOperation(
    const OBJECT& object, FUNCTION function ) {
    return OperationArg0<OBJECT, FUNCTION>( object, function );
  }

  template <class OBJECT, class FUNCTION, class ARG1>
  OperationArg1<OBJECT, FUNCTION, ARG1> MakeOperation(
    const OBJECT& object, FUNCTION function,
    Operation::Value<ARG1> arg1 ) {
    return OperationArg1<OBJECT, FUNCTION, ARG1>(
      object, function, arg1 );
  }

  template <class OBJECT, class FUNCTION, class ARG1, class ARG2>
  OperationArg2<OBJECT, FUNCTION, ARG1, ARG2> MakeOperation(
    const OBJECT& object, FUNCTION function,
    Operation::Value<ARG1> arg1, Operation::Value<ARG2> arg2 ) {
    return OperationArg2<OBJECT, FUNCTION, ARG1, ARG2>(
      object, function, arg1, arg2 );
  }

  template <class OBJECT, class FUNCTION, class ARG1, class ARG2, class ARG3>
  OperationArg3<OBJECT, FUNCTION, ARG1, ARG2, ARG3> MakeOperation(
    const OBJECT& object, FUNCTION function,
    Operation::Value<ARG1> arg1, Operation::Value<ARG2> arg2, Operation::Value<ARG3> arg3 ) {
    return OperationArg3<OBJECT, FUNCTION, ARG1, ARG2, ARG3>(
      object, function, arg1, arg2, arg3 );
  }

  template <class OBJECT, class FUNCTION, class ARG1, class ARG2, class ARG3, class ARG4>
  OperationArg4<OBJECT, FUNCTION, ARG1, ARG2, ARG3, ARG4> MakeOperation(
    const OBJECT& object, FUNCTION function,
    Operation::Value<ARG1> arg1, Operation::Value<ARG2> arg2, Operation::Value<ARG3> arg3, Operation::Value<ARG4> arg4 ) {
    return OperationArg4<OBJECT, FUNCTION, ARG1, ARG2, ARG3, ARG4>(
      object, function, arg1, arg2, arg3, arg4 );
  }

}

#endif // TT_SQUIRREL_OPERATION_H_
