#include "tt_util.h"

#include "tt_squirrel_native_api.h"
#include "tt_squirrel_utility.h"
#include "tt_squirrel_exception.h"

#include "tt_squirrel_virtual_machine.h"

using namespace TtSquirrel;


VirtualMachine::VirtualMachine( int stack_size ) :
vm_( NativeAPI::Open( stack_size ) ),
need_release_( true )
{
  Native().Std().SetErrorHandlers();
  Native().SetPrintFunction( TtSquirrel::DefaultPrintFunc );
}

VirtualMachine::VirtualMachine( HSQUIRRELVM vm, bool need_release ) :
vm_( vm ),
need_release_( need_release )
{
}

VirtualMachine::~VirtualMachine()
{
  if ( need_release_ ) {
    this->Native().Close();
  }
}

HSQUIRRELVM
VirtualMachine::GetHandle( void )
{
  return vm_;
}


NativeAPI
VirtualMachine::Native( void )
{
  return NativeAPI( vm_ );
}

// -- bundle -------------------------------------------------------------
void
VirtualMachine::GetByString( int index, const std::string& key )
{
  Native().PushString( key );
  try {
    Native().Get( Pushed( index, 1 ) );
  }
  catch ( NativeAPIException ) {
    throw GetOperationException( "string : " + key );
  }
}


void
VirtualMachine::GetByInteger( int index, int key )
{
  Native().PushInteger( key );
  try {
    Native().Get( Pushed( index, 1 ) );
  }
  catch ( NativeAPIException ) {
    throw GetOperationException( "integer : " + TtUtil::ToStringFrom( key ) );
  }
}


void
VirtualMachine::PushObject( Object& object )
{
  Native().PushObject( object.GetHandle() );
}


// -- complete set -------------------------------------------------------
namespace {
  class StackRecoverer {
  public:
    explicit StackRecoverer( NativeAPI native, unsigned int leave = 0 ) :
    native_( native ), top_( native.GetTop() + leave ) {}
    ~StackRecoverer() { native_.SetTop( top_ ); }
  private:
    NativeAPI native_;
    int       top_;
  };
}


void
VirtualMachine::NewSlot( const Operation& table, const Operation& key, const Operation& value )
{
  StackRecoverer recover( Native() );
  table();
  if ( NOT( Native().GetType( TtSquirrel::StackTop ).IsMakeableSlot() ) ) {
    throw ObjectHasNotSlotException( Native().GetType( TtSquirrel::StackTop ) );
  }
  key();
  value();
  Native().NewSlot( Pushed( TtSquirrel::StackTop, 2 ), false );
}

void
VirtualMachine::Set( const Operation& table, const Operation& key, const Operation& value )
{
  StackRecoverer recover( Native() );
  table();
  key();
  value();
  Native().Set( Pushed( TtSquirrel::StackTop, 2 ) );
}

void
VirtualMachine::Get( const Operation& table, const Operation& key )
{
  StackRecoverer recover( Native(), 1 );
  table();
  key();
  Native().Get( Pushed( TtSquirrel::StackTop, 1 ) );
  Native().Remove( Pushed( TtSquirrel::StackTop, 1 ) );
}

void
VirtualMachine::GetByString( const Operation& table, const std::string& key )
{
  StackRecoverer recover( Native(), 1 );
  table();
  Native().PushString( key );
  Native().Get( Pushed( TtSquirrel::StackTop, 1 ) );
  Native().Remove( Pushed( TtSquirrel::StackTop, 1 ) );
}

void
VirtualMachine::GetByInteger( const Operation& table, int key )
{
  StackRecoverer recover( Native(), 1 );
  table();
  Native().PushInteger( key );
  Native().Get( Pushed( TtSquirrel::StackTop, 1 ) );
  Native().Remove( Pushed( TtSquirrel::StackTop, 1 ) );
}

void
VirtualMachine::DoFile( const std::string& filename, bool print_error )
{
  StackRecoverer recover( Native() );

  Native().PushRootTable();
  try {
    Native().Std().DoFile( filename.c_str(), false, print_error );
  }
  catch ( NativeAPIException& e ) {
    throw FileExecutionException( filename );
  }
}


Object
VirtualMachine::GetStackObject( int index )
{
  return Object( *this, index );
}
