#ifndef TT_COMMAND_LINE_ANALYZER_H_
#define TT_COMMAND_LINE_ANALYZER_H_

#include <string>
#include <vector>
#include <set>
#include <limits>

// -- TtCommandLineOption ------------------------------------------------
class TtCommandLineOption {
  friend class TtCommandLineOptionAnalyzer;

public:
  explicit TtCommandLineOption( const std::string& main_option,
                                unsigned int       operand_count,
                                bool               can_duplicate,
                                bool               is_deniable,
                                bool               is_indispensable );
  virtual ~TtCommandLineOption();

  const std::string& GetMainOption( void ) const;
  void AddOptionString( const std::string& option );
  const std::set<std::string>& GetOptionStrings( void ) const;

  void SetCanDuplicate( bool flag );
  bool CanDuplicate( void ) const;

  void SetDeniable( bool flag );
  bool IsDeniable( void ) const;

  void SetIndispensable( bool flag );
  bool IsIndispensable( void ) const;

  bool IsSpecified( void ) const;
  bool IsDenied( void ) const;

  const std::vector<std::string>& GetOperand( void ) const;

  int ConvertOperandAsInteger( unsigned int index,
                               int min = std::numeric_limits<int>::min(),
                               int max = std::numeric_limits<int>::max() ) const;
  double ConvertOperandAsDouble( unsigned int index,
                                 double min = std::numeric_limits<int>::min(),
                                 double max = std::numeric_limits<int>::max() ) const;

private:
  const std::string        main_option_;
  std::set<std::string>    option_strings_;
  std::vector<std::string> operand_;
  unsigned int             operand_count_;
  bool                     can_duplicate_;
  bool                     is_deniable_;
  bool                     is_indispensable_;
  bool                     is_specified_;
  bool                     is_denied_;
};

// -- TtCommandLineOptionBoolean -----------------------------------
class TtCommandLineOptionBoolean : public TtCommandLineOption {
public:
  explicit TtCommandLineOptionBoolean( const std::string& option, bool is_indispensable = false );

  bool GetResult( void ) const;
};

// -- TtCommandLineOptionSingleOperand -----------------------------------
class TtCommandLineOptionSingleOperand : public TtCommandLineOption {
public:
  explicit TtCommandLineOptionSingleOperand( const std::string& option, bool is_indispensable = false );

  const std::string& GetSingleOperand( void ) const;

  int ConvertSingleOperandAsInteger( int min = std::numeric_limits<int>::min(),
                                     int max = std::numeric_limits<int>::max() ) const;
  double ConvertSingleOperandAsDouble( double min = std::numeric_limits<int>::min(),
                                       double max = std::numeric_limits<int>::max() ) const;
};

// -- TtCommandLineOptionAnalyzer ----------------------------------------
class TtCommandLineOptionAnalyzer {
public:
  explicit TtCommandLineOptionAnalyzer( void );
  ~TtCommandLineOptionAnalyzer();

  void Regist( TtCommandLineOption& option );

  std::vector<std::string> AnalyzeForConsole( unsigned int argc, const char* const* argv );
  std::vector<std::string> AnalyzeForWindows( const std::string& args_string );

private:
  std::vector<std::string> Analyze( const std::vector<std::string> args );

private:
  std::set<TtCommandLineOption*> options_;
};


#include "tt_exception.h"

// -- TtCommandLineOptionException ---------------------------------------
class TtCommandLineOptionException : public TtException {
};

// dpp
// IvVێp
class TtWithCommandLineOptionString
{
protected:
  explicit TtWithCommandLineOptionString( const std::string& option ) : option_( option ) {}
  const std::string& GetOption( void ) const { return option_; }
private:
  const std::string option_;
};

// Iyhێp
class TtWithCommandLineOperandString
{
protected:
  explicit TtWithCommandLineOperandString( const std::string& operand ) : operand_( operand ) {}
  const std::string& GetOperand( void ) const { return operand_; }
private:
  const std::string operand_;
};

// -- TtCommandLineOptionOperandCountException ----------------------------------
class TtCommandLineOptionOperandCountException : public TtCommandLineOptionException,
                                                 public TtWithCommandLineOptionString {
public:
  explicit TtCommandLineOptionOperandCountException( const std::string& option );

  using TtWithCommandLineOptionString::GetOption;
};

// -- TtCommandLineOptionDuplicateDefinitionException ---------------------
class TtCommandLineOptionDuplicateDefinitionException : public TtCommandLineOptionException,
                                                        public TtWithCommandLineOptionString {
public:
  explicit TtCommandLineOptionDuplicateDefinitionException( const std::string& option );

  using TtWithCommandLineOptionString::GetOption;
};

// -- TtCommandLineOptionIndispensableIsNoneException ---------------------
class TtCommandLineOptionIndispensableIsNoneException : public TtCommandLineOptionException,
                                                        public TtWithCommandLineOptionString {
public:
  explicit TtCommandLineOptionIndispensableIsNoneException( const std::string& option );

  using TtWithCommandLineOptionString::GetOption;
};

// -- TtCommandLineOptionUnknownOptionSpecifiedException ------------------
class TtCommandLineOptionUnknownOptionSpecifiedException : public TtCommandLineOptionException {
public:
  explicit TtCommandLineOptionUnknownOptionSpecifiedException( const std::string& specified_string );

  const std::string& GetSpecifiedString( void ) const;

private:
  const std::string specified_string_;
};

// -- TtCommandLineOptionDuplicatedSpecifyException -----------------------
class TtCommandLineOptionDuplicatedSpecifyException : public TtCommandLineOptionException,
                                                      public TtWithCommandLineOptionString {
public:
  explicit TtCommandLineOptionDuplicatedSpecifyException( const std::string& option );

  using TtWithCommandLineOptionString::GetOption;
};


// -- TtCommandLineOptionOperandTypeException ----------------------------
class TtCommandLineOptionOperandTypeException : public TtCommandLineOptionException,
                                                public TtWithCommandLineOptionString,
                                                public TtWithCommandLineOperandString {
public:
  explicit TtCommandLineOptionOperandTypeException( const std::string& option,
                                                    const std::string& operand,
                                                    const std::string& require_type );

  using TtWithCommandLineOptionString::GetOption;
  using TtWithCommandLineOperandString::GetOperand;

  const std::string& GetRequireType( void ) const;

private:
  const std::string require_type_;
};

// -- TtCommandLineOptionOperandRangeException ----------------------------
template <class TYPE>
class TtCommandLineOptionOperandRangeException : public TtCommandLineOptionException,
                                                 public TtWithCommandLineOptionString,
                                                 public TtWithCommandLineOperandString {
public:
  explicit TtCommandLineOptionOperandRangeException( const std::string& option,
                                                     const std::string& operand,
                                                     TYPE min, TYPE max ) :
  TtWithCommandLineOptionString( option ),
  TtWithCommandLineOperandString( operand ),
  min_( min ), max_( max ) {}

  using TtWithCommandLineOptionString::GetOption;
  using TtWithCommandLineOperandString::GetOperand;

  const TYPE& GetMinimum( void ) const { return min_; }
  const TYPE& GetMaximum( void ) const { return max_; }

private:
  TYPE min_;
  TYPE max_;
};


#endif // TT_COMMAND_LINE_ANALYZER_H_
