#ifndef TT_ENUM_H_
#define TT_ENUM_H_

#include <string>
#include <map>


template <class TYPE> class TtEnum;

// -- EnumTable --------------------------------------------------------
template <class TYPE>
class TtEnumTable {
  typedef typename TYPE::Value ValueType;

public:
  explicit TtEnumTable( void ) {}

  std::string ValueToName( ValueType value ) {
    std::map<ValueType, std::string>::iterator it = value_to_name_.find( value );
    if ( it == value_to_name_.end() ) {
      return value_to_name_[ValueType( 0 )];
    }
    return it->second;
  }

  ValueType NameToValue( const std::string& name ) {
    std::map<std::string, ValueType>::iterator it( name_to_value_.find( name ) );
    if ( it == name_to_value_.end() ) {
      return ValueType( ValueType( 0 ) );
    }
    return it->second;
  }

  void Regist( ValueType value, const std::string& name ) {
    value_to_name_[value] = name;
    name_to_value_[name] = value;
  }

private:
  std::map<ValueType, std::string> value_to_name_;
  std::map<std::string, ValueType> name_to_value_;
};

// -- TtEnumBase ---------------------------------------------------------
template <class TYPE>
class TtEnumBase {
  typedef typename TYPE::Value ValueType;

private:
  static TtEnumTable<TYPE> TABLE;

public:

  TtEnumBase( void ) : value_( ValueType( 0 ) ) {}
  TtEnumBase( ValueType value ) : value_( value ) {}

  static ValueType Parse( const std::string& name ) {
    return TABLE.NameToValue( name );
  }

  ValueType ToValue( void ) const {
    return value_;
  }

  operator int( void ) const {
    return value_;
  }

  std::string ToString( void ) const {
    return TABLE.ValueToName( value_ );
  }

  friend bool operator ==( TtEnum<TYPE> l, TtEnum<TYPE> r ) { return l.value_ == r.value_; }
  friend bool operator !=( TtEnum<TYPE> l, TtEnum<TYPE> r ) { return NOT( l == r ); }

private:
  ValueType value_;
};

// -- TtEnum -------------------------------------------------------------
template <class TYPE>
class TtEnum : public TYPE, public TtEnumBase<TYPE> {
public:
  TtEnum<TYPE>( void ) : TtEnumBase<TYPE>() {}
  TtEnum<TYPE>( typename TYPE::Value value ) : TtEnumBase<TYPE>( value ) {}
  explicit TtEnum<TYPE>( int value ) : TtEnumBase<TYPE>( static_cast<typename TYPE::Value>( value ) ) {}
};


// // Usage
// class Foo {
// public:
//   enum Value {
//     VAL1,
//     VAL2,
//     VAL3,
//   };
// };
//
// // 
// template <>
// TtEnumTable<Foo>::TtEnumTable( void ) {
// #define REGIST( NAME ) this->Regist( Foo::NAME, #NAME )
//   REGIST( VAL1 );
//   REGIST( VAL2 );
//   REGIST( VAL3 );
// #undef REGIST
// }
// 
// template <>
// TtEnumTable<Foo>
// TtEnumBase<Foo>::TABLE;

#endif // TT_ENUM_H_
