#ifndef TT_SHARED_POINTER_H_
#define TT_SHARED_POINTER_H_


// -- SharedCounter ----------------------------------------------------
class SharedCounter {
public:
  explicit SharedCounter( void ) : count_( 1 ) {
  }

  virtual ~SharedCounter() {
  }

  void AddReference( void ) {
    ++count_;
  }

  void Release( void ) {
    --count_;
    if ( count_ == 0 ) {
      this->Dispose();
      delete this;
    }
  }

  int GetCount( void ) const {
    return count_;
  }

  virtual void Dispose( void ) = 0;

private:
  int count_;
};

template <class DETAIL>
class DefaultSharedCounter : public SharedCounter {
public:
  explicit DefaultSharedCounter( DETAIL* pointer ) :
  pointer_( pointer ) {
  }

  void Dispose( void ) {
    delete pointer_;
  }

private:
  DETAIL* pointer_;
};

template <class DETAIL, class DELETOR>
class SharedCounterWithDeletor : public SharedCounter {
public:
  explicit SharedCounterWithDeletor( DETAIL* pointer, DELETOR deletor ) :
  pointer_( pointer ), deletor_( deletor ) {
  }

  void Dispose( void ) {
    deletor_( pointer_ );
  }

private:
  DETAIL* pointer_;
  DELETOR deletor_;
};


// -- TtSharedPointer ----------------------------------------------------
template <class TYPE>
class TtSharedPointer {
  template <class DETAIL> friend class TtSharedPointer;

public:
  explicit TtSharedPointer( void ) :
  pointer_( 0 ), counter_( 0 ) {
  }

  template <class DETAIL>
  explicit TtSharedPointer( DETAIL* pointer ) :
  pointer_( pointer ), counter_( new DefaultSharedCounter<DETAIL>( pointer ) ) {
  }

  template <class DETAIL, class DELETOR>
  explicit TtSharedPointer( DETAIL* pointer, DELETOR deletor ) :
  pointer_( pointer ), counter_( new SharedCounterWithDeletor<DETAIL, DELETOR>( pointer, deletor ) ) {
  }

  TtSharedPointer( const TtSharedPointer<TYPE>& other ) :
  pointer_( 0 ), counter_( 0 ) {
    this->operator =<DETAIL>( other );
  }

  template <class DETAIL>
  TtSharedPointer( const TtSharedPointer<DETAIL>& other ) :
  pointer_( 0 ), counter_( 0 ) {
    this->operator =( other );
  }

  TtSharedPointer<TYPE>& operator =( const TtSharedPointer<TYPE>& other ) {
    return this->operator =<DETAIL>( other );
  }

  template <class DETAIL>
  TtSharedPointer<TYPE>& operator =( const TtSharedPointer<DETAIL>& other ) {
    if ( other.counter_ != counter_ ) {
      if ( other.counter_ ) {
        other.counter_->AddReference();
      }
      if ( counter_ ) {
        counter_->Release();
      }
      pointer_ = other.pointer_;
      counter_ = other.counter_;
    }
    return *this;
  }

  ~TtSharedPointer() {
    if ( counter_ ) {
      counter_->Release();
    }
  }

  TYPE& operator *( void ) const {
    return *pointer_;
  }

  TYPE* operator ->( void ) const {
    return pointer_;
  }

  bool operator ==( const TtSharedPointer& other ) const {
    return pointer_ == other.pointer_;
  }

  TYPE* GetPointer( void ) const {
    return pointer_;
  }

  TYPE& GetReference( void ) const {
    return *pointer_;
  }

#ifdef DEBUG
  unsigned int GetCount( void ) const {
    return counter_ ? counter_->GetCount() : 0;
  }
#endif
  
private:
  TYPE*          pointer_;
  SharedCounter* counter_;
};


// -- TtSharedArray ------------------------------------------------------
template <class TYPE>
class TtSharedArray {
private:
  class ArrayDeletor {
  public:
    void operator ()( TYPE* pointer_ ) {
      delete[] pointer_;
    }
  };

public:

  explicit TtSharedArray( unsigned int count ) :
  size_( sizeof( TYPE ) * count ),
  pointer_( new TYPE[size_], ArrayDeletor() ) {
  }

  template <class DELETOR>
  explicit TtSharedArray( unsigned int count, DELETOR deletor ) :
  size_( sizeof( TYPE ) * count ),
  pointer_( new TYPE[size_], deletor ) {
  }

  unsigned int GetCount( void ) const {
    return size_ / sizeof( TYPE );
  }

  unsigned int GetSize( void ) const {
    return size_;
  }

  TYPE* GetPointer( void ) const {
    return pointer_.GetPointer();
  }

  TYPE& operator []( unsigned int index ) {
    return pointer_.GetPointer()[index];
  }

  const TYPE& operator []( unsigned int index ) const {
    return pointer_.GetPointer()[index];
  }

  bool operator ==( const TtSharedArray& other ) const {
    return pointer_ == other.pointer_;
  }

protected:
  unsigned int          size_;
  TtSharedPointer<TYPE> pointer_;
};


template <class TYPE>
class NoDeleting {
public:
  void operator ()( TYPE* pointer_ ) {
    NOT_USE_VAR( pointer_ );
    // do noting
  }
};


#include <string>

class TtSharedString : public TtSharedArray<char> {
public:
  explicit TtSharedString( unsigned int size );
  explicit TtSharedString( const std::string& str );

  void ExtendSize( unsigned int increasing );

  std::string ToString( void );
};

#endif // TT_SHARED_POINTER_H_
